(* $Id: compose.ml,v 1.5 2014/02/27 16:36:36 fclement Exp $ *)

open Ppm;;
open Pixmap;;

let usage_msg = "\
     Usage: \"compose -red <file-red>.ppm -green <file-green>.ppm \
    -blue <file-blue>.ppm -o <file>.ppm\"\
    \ncomposes the given Red, Green and Blue channels into the RGB image \
    <file>.ppm."
;;

let usage () = prerr_endline usage_msg;;

let compose_error s =
  prerr_endline (Printf.sprintf "compose: %s." s);
  usage ();
  exit 2
;;

(* Abstract reference with unique assignment. *)
let make_ref_unique name default =
  let set = ref false in
  let r = ref default in
  (fun () ->
    if !set then !r else
    compose_error (Printf.sprintf "option %s has not been set" name)),
  (fun x ->
    if not !set then begin r := x; set := true end else
    compose_error (Printf.sprintf "option %s has already been set" name));;

let get_rfname, set_rfname = make_ref_unique "-red" ""
and get_gfname, set_gfname = make_ref_unique "-green" ""
and get_bfname, set_bfname = make_ref_unique "-blue" ""
and get_fname, set_fname = make_ref_unique "-o" ""
;;

let parse_args () =
  Arg.parse [
    ("-red", Arg.String set_rfname,
     "<file-red>.ppm:\
      \nset the filename of the red pixmap for the red channel.");
    ("-green", Arg.String set_gfname,
     "<file-green>.ppm:\
      \nset the filename of the green pixmap for the green channel.");
    ("-blue", Arg.String set_bfname,
     "<file-blue>.ppm:\
      \nset the filename of the blue pixmap for the blue channel.");
    ("-o", Arg.String set_fname,
     "<file>.ppm:\
      \nset the filename of the output RGB pixmap.");
   ]
   (fun _ -> compose_error "no anonymous argument")
   usage_msg
;;

let compose_rgb_channels () =
  let rfname = get_rfname () in
  let gfname = get_gfname () in
  let bfname = get_bfname () in
  let fname = get_fname () in
  let pxr = load_ppm rfname in
  let pxg = load_ppm gfname in
  let pxb = load_ppm bfname in
  let px = rgb_pixmap_of_rgb_channels pxr pxg pxb in
  save_ppm fname px
;;

let main () =
  parse_args ();
  compose_rgb_channels ()
;;

main ();;
