CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: scan_utils.fpp,v 1.3 2014/03/13 13:16:38 fclement Exp $

C     
C     Primitives for scanning.
C     
      
#include "pio_constants.h"
      
      subroutine verify_scan_current_char(sbr, c)
C     Verify that scan_current_char is equal to c.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
C     Local variable.
      character cur
      
C     External functions.
      logical pio_get_debug_mode
      character get_scan_current_char
      
      cur = get_scan_current_char()
      if (cur .ne. c) then
         call error_char_syntax(sbr, c, cur)
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ': ',
     +        'character ''', c, ''' read.'
      endif
      
      return
      end
      
      
      subroutine advance_and_verify_scan_current_char(sbr, c)
C     Verify that scan_current_char is equal to c,
C     advance scan_current_char.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
      call advance_and_fill_scan_current_char(sbr)
      call verify_scan_current_char(sbr, c)
      
      return
      end
      
      
      subroutine verify_scan_current_char_and_advance(sbr, c)
C     Verify that scan_current_char is equal to c,
C     advance scan_current_char.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
      call verify_scan_current_char(sbr, c)
      call advance_and_fill_scan_current_char(sbr)
      
      return
      end
      
      
      subroutine verify_scan_current_string(sbr, s, len)
C     Verify that scan_current_char and next (len - 1) chars are equal
C     to s(1:len).
      
      implicit none
      
C     Input arguments.
      character sbr*(*), s
      integer len
      
C     Local variable.
      integer i
      
      do i = 1, len - 1
         call verify_scan_current_char_and_advance(sbr, s(i:i))
      enddo
      
      call verify_scan_current_char(sbr, s(len:len))
      
      return
      end
      
      
      subroutine verify_scan_current_string_and_advance(sbr, s, len)
C     Verify that scan_current_char and next (len - 1) chars are equal
C     to s(1:len) and advance.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), s
      integer len
      
      call verify_scan_current_string(sbr, s, len)
      
      call advance_and_fill_scan_current_char(sbr)
      
      return
      end
      
      
      subroutine verify_scan_next_line(sbr, ic, s, len)
C     Read a line and verify that it is equal to s(1:len).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*), s
      integer ic, len
      
C     Local variable.
      integer i
      
C     External function.
      logical pio_get_debug_mode
      
      call fill_buffer(sbr, ic)
      
      i = buffer_lim(PIO_IN) - 1
      if (buffer(PIO_IN)(1:i) .ne. s(1:len)) then
         call error_wrong_current_line
     +        (sbr, s(1:len), buffer(PIO_IN)(1:i))
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ': ',
     +        'line |', s(1:len), '| read.'
      endif
      
      return
      end
      
      
      subroutine verify_scan_empty_line(sbr, ic)
C     Read a line and verify that it is empty, ie contains only spaces.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer ic
      
C     Local variable.
      integer lim
      
C     External function.
      logical pio_get_debug_mode
      
      call fill_buffer(sbr, ic)
      
      lim = buffer_lim(PIO_IN)
      if (lim .gt. 1) then
         call error_non_empty_line(sbr, buffer(PIO_IN)(1:lim))
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ': ',
     +        'empty line read.'
      endif
      
      return
      end
      
      
      integer function scan_delimited_ident(sbr)
C     Scan a delimited ident.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     External function.
      integer fill_identbuf
      
      call verify_scan_current_char_and_advance(sbr, '"')
      scan_delimited_ident = fill_identbuf(sbr)
      call verify_scan_current_char(sbr, '"')
      
      return
      end
      
      
      integer function scan_delimited_characters(sbr)
C     Scan a delimited string of characters.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     External function.
      integer fill_stringbuf
      
      call verify_scan_current_char_and_advance(sbr, '"')
      scan_delimited_characters = fill_stringbuf(sbr)
      call verify_scan_current_char_and_advance(sbr, '"')
      
      return
      end
      
      
      subroutine scan_string(sbr, len)
C     Scan a character string.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer len
      
C     Local variable.
      integer sz
      
C     External function.
      integer scan_delimited_characters
      
      sz = scan_delimited_characters(sbr)
      
      if (sz .ne. len) then
         call error_wrong_string_length(sbr, len, sz)
      endif
      
      return
      end
      
      
      integer function scan_natural(sbr)
C     Scan a natural number (a string of decimal digits).
C     This routine assumes that the buffer is already filled.
C     At the end, we have either:
C      - current char is invalid, i.e. end of buffer,
C      - current char is valid and not a decimal digit.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      logical empty_token
      character c
      integer res, d
      
C     External functions.
      logical is_decimal_digit, end_of_buffer, pio_get_debug_mode
      character get_scan_current_char
      integer char_to_int
      
      empty_token = .True.
      res = 0
      
   10 continue
      
      c = get_scan_current_char()
C     Some compilers may complain, but there is no precision loss here
C     since d should be a decimal digit.
      d = char_to_int(c)
      
      if (is_decimal_digit(d)) then
         
         empty_token = .False.
         res = 10 * res + d - ASCII0
         
         if (end_of_buffer(PIO_IN)) goto 20
         
         call advance_and_fill_scan_current_char(sbr)
         goto 10
         
      endif
      
   20 continue
      
      if (empty_token) then
         call error_empty_token(sbr)
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'natural read is ', res
      endif
      
      scan_natural = res
      
      return
      end
      
      
      subroutine scan_natural_list(sbr, n, lst)
C     Scan a list of natural and store them into array lst.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n, lst(MAX_ARRAY_DIMENSION)
      
C     Local variables.
      character cur
      integer i, item
      
C     External functions.
      character get_scan_current_char
      integer scan_natural
      
      do i = 1, n
         item = scan_natural(sbr)
         lst(i) = item
         
         cur = get_scan_current_char()
         
         if (cur .eq. ',') then
            call verify_scan_current_string_and_advance(sbr, ', ', 2)
         else if (i .eq. n) then
            goto 10
         else
            call error_length_too_short(sbr, n, i)
         endif
      enddo
      
   10 continue
      
      return
      end
      
      
      character function scan_sign(sbr)
C     Scan the optional sign of an integer.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variable.
      character sgn
      
C     External function.
      character get_scan_current_char
      
      sgn = get_scan_current_char()
      if ((sgn .eq. '-') .or. (sgn .eq. '+')) then
         call advance_and_fill_scan_current_char(sbr)
      endif
      
      scan_sign = sgn
      
      return
      end
      
      
      integer function scan_int(sbr)
C     Scan an integer number.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      character sgn
      integer n
      
C     External functions.
      character scan_sign
      integer scan_natural
      
      sgn = scan_sign(sbr)
      n = scan_natural(sbr)
      
      if (sgn .eq. '-') then
         scan_int = -n
      else
         scan_int = n
      endif
      
      return
      end
      
      
      FLOAT function scan_ascii_float()
C     Scan an ascii float number either from the scanning buffer
C     buffer(PIO_IN) or from the float number scanning buffer floatbuf,
C     according to the boolean in_scanbuf.
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     External function.
      logical pio_get_debug_mode
      
      read(floatbuf, SCAN_FLOAT_FORMAT) scan_ascii_float
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        '[Pio/Fortran] ASCII float read is ', scan_ascii_float
      endif
      
      return
      end
      
      
      FLOAT function scan_binary_float(sbr)
C     Scan a binary float number from the scanning buffer buffer(PIO_IN).
      
      implicit none
      
      include 'buffers_definition.fh'
      
C     Input argument.
      character sbr*(*)
      
C     Local variables.
      integer sz, i,j
      
C     External functions.
      logical pio_get_debug_mode
      integer scan_size
      FLOAT string_to_float_bin
      
      call verify_scan_current_char_and_advance(sbr, '&')
      sz = scan_size()
      if (sz .ne. FLOAT_SIZE) then
         call error_wrong_float_size(sbr, sz)
      endif
      
      i = buffer_ptr(PIO_IN)
      j = i + FLOAT_SIZE - 1
      
      scan_binary_float = string_to_float_bin(buffer(PIO_IN)(i:j))
      
      call advance_buffer_ptr(sbr, PIO_IN, FLOAT_SIZE)
      call fill_scan_current_char(sbr)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        '[Pio/Fortran] Binary float read is ', scan_binary_float
      endif
      
      return
      end
